package com.fuzamei.web;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.security.MessageDigest;
import java.util.LinkedHashMap;
import java.util.Map;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.DigestUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.fastjson.JSON;
import com.fuzamei.constant.HintMSG;
import com.fuzamei.constant.Role;
import com.fuzamei.entity.UserDetail;
import com.fuzamei.service.AccountOpenService;
import com.fuzamei.service.AttachmentService;
import com.fuzamei.service.PromptLetterService;
import com.fuzamei.service.UserAuthoricationService;
import com.fuzamei.theromus.utils.HexUtils;
import com.fuzamei.utils.HashCodeUtil;
import com.fuzamei.utils.HashXiZhiUtil;
import com.fuzamei.utils.JSONUtil;
import com.fuzamei.utils.PageDTO;
import com.fuzamei.utils.ValidationUtil;

/**
 * 提示函模块类
 * 
 * @param file
 * @author chb & ylx
 * @throws IOException
 */
@Controller
@RequestMapping(value = "/prompt_letter")
public class PromptLetterAction {
	@Autowired
	private PromptLetterService promptLetterService;// 注入service

	private static final String SP = File.separator;
	
	@Autowired
	private UserAuthoricationService userAuthoricationService;

	@Autowired
	private AttachmentService attachmentService;
	
	private static final int ROW_NUM = 10; // 分页每页显示数据的数量
	
	/**
	 * 风险提示函发函 上传
	 * 
	 * @param file
	 * @param request
	 * @param prompt_letter_id
	 * @return
	 * @throws IOException
	 */
	@RequestMapping(value = "/uploadPromptLetterFile", method = RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> uploadPromptLetterFile(@RequestParam("file") MultipartFile file,
													  HttpServletRequest request, 
													  @RequestParam("receiverId") Object rId,
													  @RequestParam("userId") Object uId) throws IOException {
		try {
			// 非空校验
			if (file.isEmpty()) {
				throw new RuntimeException("上传文件不能为空");
			}
			int userId = ValidationUtil.checkAndAssignInt(uId);		//对用户id进行校验
			int receiverId = ValidationUtil.checkAndAssignInt(rId);	//对接受用户id进行校验
			UserDetail userDetailOfVenture = userAuthoricationService.checkUserAuthority(userId, Role.VENTURE_MANAGEMENT_POS);//获取操作用户详细信息和权限判断
			UserDetail userDetailOfAdmin = userAuthoricationService.checkUserAuthority(receiverId, Role.ADMIN);//获取接收人详细信息和权限判断

			String path = request.getServletContext().getRealPath("")+SP+"content"; //================================>>待定TODO
			
			String fileName = file.getOriginalFilename();
			String pathFile = path + SP + fileName;
			File newfile = new File(pathFile);
			//======文件开始上传=======
			File dir = new File(path);
			if (!dir.exists()) {
				dir.mkdirs();
			}
			file.transferTo(newfile);
			//======文件上传结束=======
			String hash = HashXiZhiUtil.getMD5Checksum(pathFile);// 上传（文件）之后产生哈希值====================>>待定TODO
			//将文件暂时删除，防止产生垃圾数据
			newfile.delete();
			
			int prompt_letter_id = attachmentService.generateAtachmentId();//产生一条不重复的附件id号
			
			Map<String, Object> mapToService = new LinkedHashMap<String, Object>();
			mapToService.put("userDetailOfVenture", userDetailOfVenture);
			mapToService.put("userDetailOfAdmin", userDetailOfAdmin);
			mapToService.put("attachmentId", prompt_letter_id);// 有问题 待改动
			mapToService.put("hash", hash);// 上传文件后产生的哈希值插入到提示函表//================================>>待定TODO
			mapToService.put("url", "/content/" + fileName);//================================>>待定TODO
			mapToService.put("attachmentName", fileName);
			promptLetterService.insertTiShiHanOrUserOrOtherTable(mapToService);// 在插入数据库
			
			//如果没有报错将文件上传上服务器
			file.transferTo(newfile);
			return JSONUtil.getJsonMap(200, true, HintMSG.UPLOAD_SUCCESS, null);
		} catch (Exception e) {
			return JSONUtil.getJsonMap(500, false, HintMSG.UPLOAD_FAIL+":"+e.getMessage(), null);
		}
	}

	/**
	 * 查询提示函 管理人接收列表
	 {
	 	"userId":"xxx",
	 	"page":"1",
	 	"startTime":"",
	 	"endTime":""
	 }
	 * @param data
	 * @return
	 */
	@RequestMapping(value = "/queryPromptLetter", method = RequestMethod.POST)
	@ResponseBody
	public Map<String, Object> selectPromptLetter(@RequestBody String data) {
		try {
			// System.out.println("管理人查询风险提示函信息列表");
			@SuppressWarnings("unchecked")
			Map<String, Object> map = JSON.parseObject(data,Map.class);
			int userId = ValidationUtil.checkAndAssignInt(map.get("userId"));
			UserDetail userDetail = userAuthoricationService.checkUserAuthority(userId,Role.ADMIN, Role.VENTURE_MANAGEMENT_POS);//获取操作用户详细信息和权限判断
			int page = ValidationUtil.checkMinAndAssignInt(map.get("page"), 1);
			long startTime = ValidationUtil.checkAndAssignDefaultLong(map.get("startTime"), 0L);
			long endTime = ValidationUtil.checkAndAssignDefaultLong(map.get("endTime"), Long.MAX_VALUE);
			Map<String, Object> mapToService = new LinkedHashMap<String, Object>();
			mapToService.put("userDetail", userDetail);
			mapToService.put("startPage", (page - 1) * ROW_NUM);
			mapToService.put("startTime", startTime);
			if(startTime<=endTime){
				mapToService.put("endTime", endTime);
			}else{
				mapToService.put("endTime", Long.MAX_VALUE);
			}
			mapToService.put("rowNum",  ROW_NUM);
			PageDTO pageDto = promptLetterService.selectPromptLetter(mapToService);
			return JSONUtil.getJsonMap(200, true, HintMSG.QUERY_SUCCESS, pageDto);
		} catch (Exception e) {
			return JSONUtil.getJsonMap(500, false, HintMSG.QUERY_FAIL+":"+e.getMessage(), null);
		}
	}


	/**
	 * 提示函下载 doc文档
	 * 
	 * @param request
	 * @param response
	 * @throws Exception
	 */
	@RequestMapping(value = "/downloadFile")
	@ResponseBody
	public Map<String, Object> downloadFile(HttpServletRequest request, 
											HttpServletResponse response,
											@RequestParam("url") String url,
											@RequestParam("userId") Object uId) throws Exception {
		try {
			int userId = ValidationUtil.checkAndAssignInt(uId);
			userAuthoricationService.checkUserAuthority(userId,Role.ADMIN, Role.VENTURE_MANAGEMENT_POS);//权限判断
			// 模拟文件，456.doc为需要下载的文件
			String fileName = request.getSession().getServletContext().getRealPath("") + url;
			// 获取输入流
			InputStream bis = new BufferedInputStream(new FileInputStream(new File(fileName)));
			// 截取文件名字
			String filename = url.substring(url.lastIndexOf("/") + 1);
			// 转码，免得文件名中文乱码
			filename = URLEncoder.encode(filename, "UTF-8");
			// 设置文件下载头
			response.addHeader("Content-Disposition", "attachment;filename=" + filename);
			// 1.设置文件ContentType类型，这样设置，会自动判断下载文件类型
			response.setContentType("multipart/form-data");
			BufferedOutputStream out = new BufferedOutputStream(response.getOutputStream());
			int len = 0;
			while ((len = bis.read()) != -1) {
				out.write(len);
				out.flush();
			}
			out.close();
			bis.close();
			// 返给前端map进行前端提示
			return JSONUtil.getJsonMap(200, true, "下载成功", null);
		} catch (Exception e) {
			return JSONUtil.getJsonMap(500, false, "下载失败:" + e.getMessage(), null);
		}
	}

}
